@@ -7,6 +7,14 @@ from ruamel.yaml import YAML
 from pathlib import Path
 import re
 import torch
+from numpy import random
+
+from yolov5.models.experimental import attempt_load
+from yolov5.utils.datasets import LoadStreams, LoadImages
+from yolov5.utils.general import check_img_size, check_requirements, check_imshow, non_max_suppression, apply_classifier, \
+    scale_coords, xyxy2xywh, strip_optimizer, set_logging, increment_path
+from yolov5.utils.plots import plot_one_box
+from yolov5.utils.torch_utils import select_device, load_classifier, time_synchronized
 
 def to_yolo(pxl_annotations_dict=None,labelmap=None, base_imgdir=None, targetdir=None, prop=0.7, copy_image=True):
     """
@@ -197,6 +205,76 @@ def tf_events_to_dict(path, type=''):
 
     return log_dict
 
+def detect(weights, path, thresh=0.7, size=640, iou_thresh=0.45, augment=True, agnostic_nms=True):
+    source, imgsz = path, size
+    # Initialize
+    set_logging()
+    device = 'cuda:0' if torch.cuda.is_available() else 'cpu'
+    half = device != 'cpu'  # half precision only supported on CUDA
+
+    # Load model
+    model = attempt_load(weights, map_location=device)  # load FP32 model
+    stride = int(model.stride.max())  # model stride
+    imgsz = check_img_size(imgsz, s=stride)  # check img_size
+    if half:
+        model.half()  # to FP16
+
+    # Second-stage classifier
+    classify = False
+    if classify:
+        modelc = load_classifier(name='resnet101', n=2)  # initialize
+        modelc.load_state_dict(torch.load('weights/resnet101.pt', map_location=device)['model']).to(device).eval()
+
+    # Set Dataloader
+    vid_path, vid_writer = None, None
+    save_img = True
+    dataset = LoadImages(source, img_size=imgsz, stride=stride)
+
+    # Get names and colors
+    names = model.module.names if hasattr(model, 'module') else model.names
+    colors = [[random.randint(0, 255) for _ in range(3)] for _ in names]
+
+    # Run inference
+    if device != 'cpu':
+        model(torch.zeros(1, 3, imgsz, imgsz).to(device).type_as(next(model.parameters())))  # run once
+    predictions = []
+    for path, img, im0s, vid_cap in dataset:
+        img = torch.from_numpy(img).to(device)
+        img = img.half() if half else img.float()  # uint8 to fp16/32
+        img /= 255.0  # 0 - 255 to 0.0 - 1.0
+        if img.ndimension() == 3:
+            img = img.unsqueeze(0)
+
+        # Inference
+        t1 = time_synchronized()
+        pred = model(img, augment=augment)[0]
+        # Apply NMS
+        pred = non_max_suppression(pred, thresh, iou_thresh, agnostic=agnostic_nms)
+        t2 = time_synchronized()
+        # Apply Classifier
+        if classify:
+            pred = apply_classifier(pred, modelc, img, im0s)
+        predictions.append(pred[0].cpu().numpy())
+        # Process detections
+        for i, det in enumerate(pred):  # detections per image
+            p, s, im0, frame = path, '', im0s, getattr(dataset, 'frame', 0)
+
+            p = Path(p)  # to Path
+            s += '%gx%g ' % img.shape[2:]  # print string
+            gn = torch.tensor(im0.shape)[[1, 0, 1, 0]]  # normalization gain whwh
+            if len(det):
+                # Rescale boxes from img_size to im0 size
+                det[:, :4] = scale_coords(img.shape[2:], det[:, :4], im0.shape).round()
+
+                # Print results
+                for c in det[:, -1].unique():
+                    n = (det[:, -1] == c).sum()  # detections per class
+                    s += f"{n} {names[int(c)]}{'s' * (n > 1)}, "  # add to string
+
+
+            # Print time (inference + NMS)
+            print(f'{s}Done. ({t2 - t1:.3f}s)')
+    return predictions
 
 def setup_hyp(exp_name, cfg, checkpoint_dir, params={}, label_map=[]):
     YOLOSIZE = cfg.split('/')[2][6]
@@ -209,7 +287,7 @@ def setup_hyp(exp_name, cfg, checkpoint_dir, params={}, label_map=[]):
     for f in tmp:
         if f.endswith('.pt'):
             weight_path = os.path.join(checkpoint_dir, f)
-        if f.endswith('.yaml'.format(exp_name)):
+        if f.endswith('.yaml'):
             hyp_path = os.path.join(checkpoint_dir, f)
     
 
@@ -318,7 +396,7 @@ def get_metrics_curves(final_run_path):
     return confusion_matrix, F1_curve, labels_correlogram, P_curve, PR_curve, R_curve
 
 def send_run_to_picsellia(experiment, YOLODIR):
-    final_run_path = find_final_run(YOLODIR, experiment.experiment_name)
+    final_run_path = find_final_run(YOLODIR, experiment.name)
     best_weigths, last_weights, hyp_yaml = get_weights_and_config(final_run_path)
 
     if best_weigths is not None:
